# R functions for estimation of 95% confidence intervals for the changepoint of treatment stratification
# and code to perform the simulation studies for Model I shown in the manuscript
#
# For each function the data frame including variables for time, status, covariate and treamtent have to be passed.
# Names of the according variables have to be passed to the function
# Example:
# changeCI.Fieller(Data=df, Timevar="Time", Statvar="Status", Xvar="X", Treatvar="Group")
#
# For questions and remarks please contact:
# Bernhard Haller
# Institute of Medical Informatics, Statistics and Epidemiology
# Technical university of Munich
# bernhard.haller@tum.de
# 

library(survival)

# Estimation following Fieller's theorem (see Section 3.1):
#
# A list is returned. 
# The first argument is a vector of length 4 showing the estimated confidence interval.
# The second argument indicates the case as presented in Figure S1 of the online supplemental file:
# 	- Case 1: Finite confidence interval as shown in Figure 1 a.
#             The first argument of the confidence interval represents the lower CI limit, the second the upper limit.
#             The third and fouth argument are given as zero - they are not relevant for that case.
#   - Case 2: Complement of a finite interval as shown in Figure 1 b.
#             The first argument of the confidence interval is minus infinity, the second argument represents the 
#             upper limit of the first part of the interval.
#             The third argument gives the lower limit of the second part of the interval, the fourth argument is infinity.
#   - Case 3: An infinite confidence interval as shown in Figure 1 c.
#             The first argument is minus infinity, the second argument is plus infinity. 
#             The third and fourth argument are not relevant for that case and both have a value of zero.

changeCI.Fieller <- function(Data, Timevar, Statvar, Xvar, Treatvar) 
{
	library(survival)
	
	COX <- coxph(Surv(Data[,Timevar], Data[,Statvar]) ~ Data[,Xvar] * Data[,Treatvar])
	
	b.treat <- COX$coef[2]
	b.int <- COX$coef[3]
	Cov <- COX$var
	
	f0 <- b.treat^2 - qnorm(0.975)^2*Cov[2,2]
	f1 <- -b.treat*b.int + qnorm(0.975)^2*Cov[2,3]
	f2 <- b.int^2 - qnorm(0.975)^2*Cov[3,3]
	
	D <- f1^2 - f0*f2
	
	if(D>=0 & f2>=0){
		r1 <- (f1 - sqrt(D)) / f2
		r2 <- (f1 + sqrt(D)) / f2
		int <- c(r1,r2,0,0)
		case <- 1}
	
	if(D>=0 & f2<0){
		r1 <- (f1 - sqrt(D)) / f2
		r2 <- (f1 + sqrt(D)) / f2
		int <- c(-Inf,r2,r1,Inf)
		#int <- c(-Inf, Inf)
		case <- 2}
	
	if(D<0 & f2<0){
		int <- c(-Inf,Inf,0,0)
		#int <- c(-Inf,Inf)
		case <- 3}
	
	return(list(Cint=int, Case=case))
} 




# Delta method (see Section 3.2.1):
#
# A vector giving the lower and upper limit of the confidence interval are returned.

changeCI.Delta <- function(Data, Timevar, Statvar, Xvar, Treatvar)
{
	library(survival)
	
	COX <- coxph(Surv(Data[,Timevar], Data[,Statvar]) ~ Data[,Xvar] * Data[,Treatvar])
	
	b.treat <- COX$coef[2]
	b.int <- COX$coef[3]
	Cov <- COX$var
	
	D.theta <- matrix(c(- 1 / b.int, b.treat/b.int^2), nrow=1)
	DeltaCov <- D.theta %*% Cov[2:3,2:3] %*% t(D.theta)
	CI.delta <- c(-b.treat/b.int - qnorm(0.975)*sqrt(DeltaCov),
								-b.treat/b.int + qnorm(0.975)*sqrt(DeltaCov))
	return(CI.delta)
}






# Bootstrap methods (see Section 3.3):


# Percentile-based and normal intervals
# Returns a list providing confidence intervals following the
#   - Percentile-based approach (Section 3.3.1, first component "Ci.boot.percentile")
#   - Normal approach (Section 3.3.2, second component "Ci.boot.normal") 

changeCI.boot <- function(Data, Timevar, Statvar, Xvar, Treatvar, bootsamps)
{
	CUT <- c()
	br <- 1
	a <- 1
	while(br <= bootsamps)
	{
		bdat <- sample(1:dim(Data)[1], dim(Data)[1], repl=T)
		bootData <- Data[bdat,]
		YY <- 0
		CoxBoot <- tryCatch(coxph(Surv(bootData[,Timevar], bootData[,Statvar]) ~ bootData[,Xvar] * bootData[,Treatvar]),
												warning=function(cond) YY <<- 1)
		
		if(YY==0) CUT[br] <- - CoxBoot$coefficients[2] / CoxBoot$coefficients[3]
		br <- br+1
		a <- a+1
		br <- br-YY
	}
	percentile <- quantile(CUT, probs=c(0.025, 0.975))
	normal <- mean(CUT) + c(-1,1)*qnorm(0.975)*sd(CUT)
	return(list(Ci.boot.percentile=percentile, Ci.boot.normal=normal)) 
}


# Wild Bootstrap (Section 3.3.3)
# Returns a list providing confidence intervals following the Wild boostrap approach (Section 3.3.3)

changeCI.wildboot <- function(Data, Timevar, Statvar, Xvar, Treatvar, bootsamps){
	Cox <- coxph(Surv(Data[,Timevar], Data[,Statvar]) ~ Data[,Xvar] * Data[,Treatvar])
	beta.hat <- Cox$coefficients
	bG <- beta.hat[2]
	bGX <- beta.hat[3]
	frac <- - bG / bGX
	
	gradient.g <- matrix(c(- 1 / sqrt(bG^2 + bGX^2), bG / bGX / sqrt(bG^2 + bGX^2) ), nr=1)
	gradient.h <- matrix(c(-1 / bGX, bG / bGX^2), nr=1)
	
	n <- length(Data[,Timevar])
	
	gamma.iid <- sapply(1:n, function(k){
		ifelse(c(Data[k,Statvar]==1, Data[k,Statvar]==1, Data[k,Statvar]==1),
					 c(Data[k,Xvar], Data[k,Treatvar], Data[k,Xvar] * Data[k,Treatvar])
					 - c(sum(ifelse(Data[,Timevar] >= Data[k,Timevar], 
					 							 Data[,Xvar] *exp(Data[,Xvar]*beta.hat[1] 
					 							 								 + Data[,Treatvar]*beta.hat[2]
					 							 								 + Data[,Xvar]*Data[,Treatvar]*beta.hat[3]),
					 							 0)), 
					 		sum(ifelse(Data[,Timevar] >= Data[k,Timevar], 
					 							 Data[,Treatvar] *exp(Data[,Xvar]*beta.hat[1] 
					 							 										 + Data[,Treatvar]*beta.hat[2]
					 							 										 + Data[,Xvar]*Data[,Treatvar]*beta.hat[3]), 
					 							 0)),
					 		sum(ifelse(Data[,Timevar] >= Data[k,Timevar], 
					 							 Data[,Xvar]*Data[,Treatvar]*exp(Data[,Xvar]*beta.hat[1] 
					 							 																+ Data[,Treatvar]*beta.hat[2]
					 							 																+ Data[,Xvar]*Data[,Treatvar]*beta.hat[3]), 
					 							 0))) / sum(ifelse(Data[,Timevar] >= Data[k,Timevar], 
					 							 									exp(Data[,Xvar]*beta.hat[1] 
					 							 											+ Data[,Treatvar]*beta.hat[2]
					 							 											+ Data[,Xvar]*Data[,Treatvar]*beta.hat[3]),
					 							 									0)),
					 c(0,0,0))
	})
	gamma.iid <- t(gamma.iid)
	good_rows <- apply(gamma.iid, 1, function(row) any(row !=0 ))
	beta.iid <- gamma.iid[good_rows,]
	
	n.eff <- dim(beta.iid)[1]
	dim.beta <- dim(beta.iid)[2]
	
	res <- numeric(bootsamps)
	
	a <- 1
	br <- 1
	while(br <= bootsamps){
		
		G <- rpois(n.eff,1) - 1
		wbs.score <- t( matrix(G, nr=1) %*% beta.iid )
		
		square <- function(x) matrix(x, nc=1) %*% matrix(x, nr=1)
		temp.Gbeta.mat <- replicate(dim.beta, G) * beta.iid
		wbs.varcov <- matrix(rowSums(apply(temp.Gbeta.mat, 1, FUN=square)), dim.beta, dim.beta)
		
		wbs.beta.hat <- 0
		wbs.beta.hat <- solve(wbs.varcov /sqrt(n)) %*% wbs.score
		if(sum(wbs.beta.hat != 0)){
			res[br] <- gradient.h %*% wbs.beta.hat[2:3]
			br <- br+1
			a <- a+1
		}
	}
	
	q <- quantile(abs(res), probs=0.95)
	Ci.wildboot <- c(-q/sqrt(n) + frac, q/sqrt(n) + frac)
	return(Ci.wildboot=Ci.wildboot) 
}



# Function to run the simulations for Model I. 
# A table providing all the parameters used in our simulations is provided in the 
# online supplemental file (Table S5).
#
# "Distribution" is the distribution of the covariate and has to be either "norm" or "unif"
# A list with the coverage proportions of all methods, median confidence interval widths 
# with 10th and 90th percentile and the proportion of infinite intervals obtained following
# Fieller's theorem is returned

Sim.M1 <- function(SimRuns, SampleSize, lam.cens, lam.0, Distribution, beta.G, beta.X, beta.G.X, boot.samples)
{
	if(!is.element(Distribution, c("norm", "unif"))) stop("'Distribution' must be either 'norm' or 'unif'")

	ciFieller <- matrix(nrow=SimRuns, ncol=4)
	ciDelta <- matrix(nrow=SimRuns, ncol=2)
	ciboot.np <- matrix(nrow=SimRuns, ncol=2)
	ciboot.par <- matrix(nrow=SimRuns, ncol=2)
	ciboot.wild <- matrix(nrow=SimRuns, ncol=2)
	
	# Estiamte confidence intervals
	for(rr in 1:SimRuns)
	{
	  G <- sample(0:1, SampleSize, repl=T)
	  if(Distribution=="norm") xx <- rnorm(SampleSize)
	  if(Distribution=="unif") xx <- runif(SampleSize, min = -0.5, max = 0.5)
	  
	  hazard <-  lam.0* exp(xx*beta.X + G*beta.G + xx*G*beta.G.X)
	  Ei <- rexp(SampleSize, hazard)
	  Ci <- rexp(SampleSize, lam.cens)
	  Ti <- pmin(Ei, Ci)
	  St <- as.numeric(Ei<Ci)
	  DATA <- data.frame(Time=Ti, Status=St, Treat=G, Xvar=xx)
	  
		ciFieller[rr,] <- changeCI.Fieller(Data = DATA, Timevar = "Time", Statvar = "Status", Xvar = "Xvar", Treatvar = "Treat")$Cint
		ciDelta[rr,] <- changeCI.Delta(Data = DATA, Timevar = "Time", Statvar = "Status", Xvar = "Xvar", Treatvar = "Treat")
		CIboot <- changeCI.boot(Data = DATA, Timevar = "Time", Statvar = "Status", Xvar = "Xvar", Treatvar = "Treat", bootsamps = boot.samples)
		ciboot.np[rr,] <- CIboot$Ci.boot.percentile
		ciboot.par[rr,] <- CIboot$Ci.boot.normal
		ciboot.wild[rr,] <- changeCI.wildboot(Data = DATA, Timevar = "Time", Statvar = "Status", Xvar = "Xvar", Treatvar = "Treat", bootsamps = boot.samples)
		cat(rr,"/",sep="")
	}	
	
	# Derive coverage
	CoverMat <- matrix(nrow=SimRuns, ncol=5)
	colnames(CoverMat) <- c("Fieller", "Delta", "Boot.perc", "Boot.norm", "Boot.wild")
	CoverMat[,"Fieller"] <- as.numeric((ciFieller[,1] < -beta.G/beta.G.X & ciFieller[,2] > -beta.G/beta.G.X) | 
																		 	(ciFieller[,3] < -beta.G/beta.G.X & ciFieller[,4] > -beta.G/beta.G.X))
	CoverMat[,"Delta"] <- as.numeric(ciDelta[,1] < -beta.G/beta.G.X & ciDelta[,2] > -beta.G/beta.G.X)
	CoverMat[,"Boot.perc"] <- as.numeric(ciboot.np[,1] < -beta.G/beta.G.X & ciboot.np[,2] > -beta.G/beta.G.X)
	CoverMat[,"Boot.norm"] <- as.numeric(ciboot.par[,1] < -beta.G/beta.G.X & ciboot.par[,2] > -beta.G/beta.G.X)
	CoverMat[,"Boot.wild"] <- as.numeric(ciboot.wild[,1] < -beta.G/beta.G.X & ciboot.wild[,2] > -beta.G/beta.G.X)
	Coverage <- apply(CoverMat, 2, mean)
	
	# Derive Width
	WidthMat <- matrix(nrow=SimRuns, ncol=5)
	colnames(WidthMat) <- c("Fieller", "Delta", "Boot.perc", "Boot.norm", "Boot.wild")
	WidthMat[,"Fieller"] <- (ciFieller[,2] - ciFieller[,1]) + (ciFieller[,4] - ciFieller[,3])
	WidthMat[,"Delta"] <- (ciDelta[,2] - ciDelta[,1])
	WidthMat[,"Boot.perc"] <- (ciboot.np[,2] - ciboot.np[,1])
	WidthMat[,"Boot.norm"] <- (ciboot.par[,2] - ciboot.par[,1])
	WidthMat[,"Boot.wild"] <- (ciboot.wild[,2] - ciboot.wild[,1])
	Width <- apply(WidthMat, 2, function(x) quantile(x, probs = c(0.5,0.1,0.9), na.rm=T))
	
	# Fieller - infinite intervals:
	Fieller.prop.inf <- mean(WidthMat[,"Fieller"]==Inf)
	cat("\n")
	
	return(list(Coverage=Coverage, Width=Width, Fieller.prop.inf=Fieller.prop.inf))
}

# A print funtion to organized the output
print.Sc.M1 <- function(SimScen)
{
	cat("Confidence interval coverage:\n")
	print(SimScen$Coverage)
	
	cat("\nConfidence interval width:\n")
	print(round(SimScen$Width, 2))
	
	cat("\nProportion of infinite intervals using Fieller's approach:", SimScen$Fieller.prop.inf,"\n")
}


# For example Scenario 15 can be initiated by
ScM1.15 <- Sim.M1(SimRuns=2000, SampleSize=1000, lam.cens=0.3, lam.0=1, Distribution="norm", beta.G=-qnorm(0.7)*log(1.4), beta.X=log(1.25), beta.G.X=log(1.4), boot.samples=1000)
print.Sc.M1(ScM1.15)