# R functions for estimation of 95% confidence intervals for the changepoint of treatment stratification
# in the presence of two further prognostic variables
# and code to perform the simulation studies for Model II shown in the manuscript
#

library(mvtnorm)
library(survival)

# Application of the method following Fieller's theorem in the presence of two further prognostic variables
Fieller.M2.k2 <- function(Data, Timevar, Treatvar, Statvar, Xvar, Zvar1, Zvar2)
{
  COX <- coxph(Surv(Data[,Timevar], Data[,Statvar]) ~ Data[,Treatvar]*Data[,Xvar] + Data[,Zvar1] + Data[,Zvar2])
  beta <- COX$coef
  Cov <- COX$var
  
  # Note that in R coefficients for main effects are given first in the vector of regression coefficients 
  # and coefficients for interaction terms follow at the end. Consequently, the ordering of regression coefficients and vectors
  # K and L is slightly different to the presentation in the manuscript
  K <- -c(1,0,0,0,0)
	L <- c(0,0,0,0,1)
	
	qn <- qnorm(0.975)
	
	A <- (L%*%beta)^2 - qn^2 * L%*%Cov%*%L
	B <- 2 * (qn^2 * K%*%Cov%*%L - (K%*%beta)*(L%*%beta))
	C <- (K%*%beta)^2 - qn^2*K%*%Cov%*%K
	
	if((B^2 - 4*A*C >= 0) & (A >= 0) )
		cif <- c((-B - (B^2 - 4*A*C)^0.5)/(2*A),
						 (-B + (B^2 - 4*A*C)^0.5)/(2*A), 0, 0)
	
	if((B^2 - 4*A*C >= 0) & (A < 0) )
		cif <- c(-Inf, (-B + (B^2 - 4*A*C)^0.5)/(2*A), 
						 (-B - (B^2 - 4*A*C)^0.5)/(2*A), Inf)
	
	if((B^2 - 4*A*C < 0) & (A < 0) )
		cif <- c(-Inf, Inf, 0, 0)
	
	return(cif)
}


# Application of the delta method in the presence of two further prognostic variables
Delta.M2.k2 <- function(Data, Timevar, Treatvar, Statvar, Xvar, Zvar1, Zvar2)
{
  COX <- coxph(Surv(Data[,Timevar], Data[,Statvar]) ~ Data[,Treatvar]*Data[,Xvar] + Data[,Zvar1] + Data[,Zvar2])
  beta <- COX$coef
  Cov <- COX$var
  
	D.theta <- c(-1/beta[5], beta[1] / beta[5]^2)
	Var.g <- c(matrix(D.theta, nrow=1) %*% Cov[c(1,5), c(1,5)] %*% matrix(D.theta, ncol=1))
	cid <- - beta[1] / beta[5] + c(-1,1)*qnorm(0.975)*sqrt(Var.g)
	return(cid)
}

# Function to run the simulations for Model II. 
# A table showing all the parameters used in our simulations is provided in the 
# online supplemental file (Table S6).
#
# A list with the coverage proportions of all methods, median confidence interval widths 
# with 10th and 90th percentile and the proportion of infinite intervals obtained following
# Fieller's theorem is returned
# Results are provide for models not considering Z_1 and Z_2 as covariates and for models 
# with consideration of the prognostic variables
#
# Please note that the functions "changeCI.Fieller" and "changeCI.Delta" given in the file "Model_I.R"
# have to be run before starting the simulation

Sim.M2 <- function(SimRuns, SampleSize, k.z, lam.cens, lam.0, beta.G, beta.X, beta.G.X, beta.Z, r)
{
  ciFieller.noz <- matrix(nrow=SimRuns, ncol=4)
  ciDelta.noz <- matrix(nrow=SimRuns, ncol=2)
  ciFieller.withz <- matrix(nrow=SimRuns, ncol=4)
  ciDelta.withz <- matrix(nrow=SimRuns, ncol=2)
  
  # Estiamte confidence intervals
  for(rr in 1:SimRuns)
  {
    G <- sample(0:1, SampleSize, repl=T)
    Sigma <- matrix(r, nrow=k.z+1, ncol=k.z+1)
    diag(Sigma) <- 1
    Covs <- rmvnorm(SampleSize, mean = rep(0, k.z + 1), sigma = Sigma)
    
    hazard <- lam.0 * exp(beta.X*Covs[,1] + beta.G*G + beta.G.X*Covs[,1]*G + Covs[,2:dim(Covs)[2]]%*%matrix(beta.Z, ncol=1))
    Ei <- rexp(SampleSize, rate = hazard)
    Ci <- rexp(SampleSize, lam.cens)
    Ti <- pmin(Ei, Ci)
    St <- as.numeric(Ei < Ci)
    
    DATA <- data.frame(Time=Ti, Status=St, Treat=G, Xvar=Covs[,1], ZVars=Covs[,-1])
    
    ciFieller.noz[rr,] <- changeCI.Fieller(Data = DATA, Timevar = "Time", Statvar = "Status", Xvar = "Xvar", Treatvar = "Treat")$Cint
    ciDelta.noz[rr,] <- changeCI.Delta(Data = DATA, Timevar = "Time", Statvar = "Status", Xvar = "Xvar", Treatvar = "Treat")
    ciFieller.withz[rr,] <- Fieller.M2.k2(Data = DATA, Timevar = "Time", Statvar = "Status", Xvar = "Xvar", Treatvar = "Treat", Zvar1 = "ZVars.1", Zvar2 = "ZVars.2")
    ciDelta.withz[rr,] <- Delta.M2.k2(Data = DATA, Timevar = "Time", Statvar = "Status", Xvar = "Xvar", Treatvar = "Treat", Zvar1 = "ZVars.1", Zvar2 = "ZVars.2")
    
    cat(rr,"/",sep="")
  }	
  
  # Derive coverage
  CoverMat <- matrix(nrow=SimRuns, ncol=4)
  colnames(CoverMat) <- c("Fieller_no_Z", "Delta_no_Z", "Fieller_with_Z", "Delta_with_Z")
  CoverMat[,"Fieller_no_Z"] <- as.numeric((ciFieller.noz[,1] < -beta.G/beta.G.X & ciFieller.noz[,2] > -beta.G/beta.G.X) | 
                                       (ciFieller.noz[,3] < -beta.G/beta.G.X & ciFieller.noz[,4] > -beta.G/beta.G.X))
  CoverMat[,"Delta_no_Z"] <- as.numeric(ciDelta.noz[,1] < -beta.G/beta.G.X & ciDelta.noz[,2] > -beta.G/beta.G.X)
  CoverMat[,"Fieller_with_Z"] <- as.numeric((ciFieller.withz[,1] < -beta.G/beta.G.X & ciFieller.withz[,2] > -beta.G/beta.G.X) | 
                                           (ciFieller.withz[,3] < -beta.G/beta.G.X & ciFieller.withz[,4] > -beta.G/beta.G.X))
  CoverMat[,"Delta_with_Z"] <- as.numeric(ciDelta.withz[,1] < -beta.G/beta.G.X & ciDelta.withz[,2] > -beta.G/beta.G.X)
  Coverage <- apply(CoverMat, 2, mean)

  # Derive Width
  WidthMat <- matrix(nrow=SimRuns, ncol=4)
  colnames(WidthMat) <- c("Fieller_no_Z", "Delta_no_Z", "Fieller_with_Z", "Delta_with_Z")
  WidthMat[,"Fieller_no_Z"] <- (ciFieller.noz[,2] - ciFieller.noz[,1]) + (ciFieller.noz[,4] - ciFieller.noz[,3])
  WidthMat[,"Delta_no_Z"] <- (ciDelta.noz[,2] - ciDelta.noz[,1])
  WidthMat[,"Fieller_with_Z"] <- (ciFieller.withz[,2] - ciFieller.withz[,1]) + (ciFieller.withz[,4] - ciFieller.withz[,3])
  WidthMat[,"Delta_with_Z"] <- (ciDelta.withz[,2] - ciDelta.withz[,1])
  Width <- apply(WidthMat, 2, function(x) quantile(x, probs = c(0.5,0.1,0.9)))
  
  # Fieller - infinite intervals:
  Fieller.prop.inf.noz <- mean(WidthMat[,"Fieller_no_Z"]==Inf)
  Fieller.prop.inf.withz <- mean(WidthMat[,"Fieller_with_Z"]==Inf)
  cat("\n")
  
  return(list(Coverage=Coverage, Width=Width, 
              Fieller.prop.inf=c(no.z=Fieller.prop.inf.noz, with.z=Fieller.prop.inf.withz)))
}

# A print funtion to organize the output
print.Sc.M2 <- function(Sim.Scen.M2)
{
  cat("Confidence interval coverage:\n")
  print(Sim.Scen.M2$Coverage)
  
  cat("\nConfidence interval width:\n")
  print(round(Sim.Scen.M2$Width, 2))
  
  cat("\nProportion of infinite intervals using Fieller's approach - without Z:", Sim.Scen.M2$Fieller.prop.inf[1])
  cat("\nProportion of infinite intervals using Fieller's approach - with Z:", Sim.Scen.M2$Fieller.prop.inf[2],"\n")
}

# For example the eleventh scenario for Model II can be initiated by:
ScM2.11 <- Sim.M2(SimRuns = 2000, SampleSize =  1000, k.z = 2, lam.cens = 0.3, lam.0 = 1, 
       					 beta.G = 0, beta.X = log(1.25), beta.G.X = log(1.4),beta.Z = c(log(1.2),log(1.2)), r = 0.5)
print.Sc.M2(ScM2.11)


