# R functions for estimation of 95% confidence intervals for the changepoint of treatment stratification
# for given values of two predictive covariates Z_1 and Z_2
# and code to perform the simulation studies for Model III shown in the manuscript
#

library(mvtnorm)
library(survival)

# Application of the method following Fieller's theorem in the presence of two further predicitve variables
# "Zseq" is a vector giving the values of Z_1 and Z_2 used for estimation of the confidence intervals
Fieller.M3.k2 <- function(Data, Timevar, Statvar, Treatvar, Xvar, Zvar1, Zvar2, Zseq)
{
	COX.intz <- coxph(Surv(Data[,Timevar], Data[,Statvar]) ~ Data[,Treatvar]*Data[,Xvar] + Data[,Treatvar]*Data[,Zvar1] + Data[,Treatvar]*Data[,Zvar2])
	beta <- COX.intz$coef
	Cov <- COX.intz$var
	
	CiMatrix.Fi <- matrix(nrow=length(Zseq)^2, ncol=6)
	colnames(CiMatrix.Fi) <- c("Z.val_1", "Z.val_2", "CI_low.1", "CI_upp.1", "CI_low.2", "CI_upp.2")
	for(i in 1:length(Zseq))
	{
		for(j in 1:length(Zseq))
		{
			# Note that in R coefficients for main effects are given first in the vector of regression coefficients 
			# and coefficients for interaction terms follow at the end. Consequently, the ordering of regression coefficients and vectors
			# K and L is slightly different to the presentation in the manuscript
			K <- -c(1,0,0,0,0,Zseq[i],Zseq[j])
			L <- c(0,0,0,0,1,0,0)
			
			qn <- qnorm(0.975)
			
			A <- (L%*%beta)^2 - qn^2 * L%*%Cov%*%L
			B <- 2 * (qn^2 * K%*%Cov%*%L - (K%*%beta)*(L%*%beta))
			C <- (K%*%beta)^2 - qn^2*K%*%Cov%*%K
			
			if((B^2 - 4*A*C >= 0) & (A >= 0) )
				cif <- c((-B - (B^2 - 4*A*C)^0.5)/(2*A),
								 (-B + (B^2 - 4*A*C)^0.5)/(2*A), 0, 0)
			
			if((B^2 - 4*A*C >= 0) & (A < 0) )
				cif <- c(-Inf, (-B + (B^2 - 4*A*C)^0.5)/(2*A), 
								 (-B - (B^2 - 4*A*C)^0.5)/(2*A), Inf)
			
			if((B^2 - 4*A*C < 0) & (A < 0) )
				cif <- c(-Inf, Inf, 0, 0)
			
			CiMatrix.Fi[(i-1)*length(Zseq)+j, 1] <- Zseq[i]
			CiMatrix.Fi[(i-1)*length(Zseq)+j, 2] <- Zseq[j]
			CiMatrix.Fi[(i-1)*length(Zseq)+j, 3:6] <- cif
		}
	}
	return(CiMatrix.Fi)
}


# Application of the delta method in the presence of two further predictive variables
# "Zseq" is a vector giving the values of Z_1 and Z_2 used for estimation of the confidence intervals
Delta.M3.k2 <- function(Data, Timevar, Statvar, Treatvar, Xvar, Zvar1, Zvar2, Zseq)
{
	COX.intz <- coxph(Surv(Data[,Timevar], Data[,Statvar]) ~ Data[,Treatvar]*Data[,Xvar] + Data[,Treatvar]*Data[,Zvar1] + Data[,Treatvar]*Data[,Zvar2])
	beta <- COX.intz$coef
	Cov <- COX.intz$var
	
	CiMatrix.De <- matrix(nrow=length(Zseq)^2, ncol=4)
	colnames(CiMatrix.De) <- c("Z.val_1", "Z.val_2", "CI_low", "CI_upp")
	cp <- c()
	for(i in 1:length(Zseq))
	{
		for(j in 1:length(Zseq))
		{
			g.theta.zseq <- - (beta[1] + Zseq[i]*beta[6] + Zseq[j]*beta[7]) / beta[5]
			D.theta.zseq <- c(-1/beta[5], (beta[1] + Zseq[i]*beta[6] + Zseq[j]*beta[7]) / beta[5]^2, -Zseq[i]/beta[5], -Zseq[j]/beta[5])
			Var.g.zseq <- c(matrix(D.theta.zseq, nrow=1) %*% Cov[c(1,5,6,7), c(1,5,6,7)] %*% matrix(D.theta.zseq, ncol=1))
			CiMatrix.De[(i-1)*length(Zseq) + j, 1] <- Zseq[i]
			CiMatrix.De[(i-1)*length(Zseq) + j, 2] <- Zseq[j]
			CiMatrix.De[(i-1)*length(Zseq) + j, 3:4] <- g.theta.zseq + c(-1,1)*qnorm(0.975)*sqrt(Var.g.zseq)
		}
	}
	return(CiMatrix.De)
}



# Function to run the simulations for Model III. 
# A table showing all the parameters used in our simulations is provided in the 
# online supplemental file (Table S7).
#
# A list with the coverage proportions of all methods, median confidence interval widths 
# with 10th and 90th percentile and the proportion of infinite intervals obtained following
# Fieller's theorem is returned.
# Results are given for each pairwise combination of the components of the vector "Zeval" that
# indicates the values of Z_1 and Z_2 for which confidence intervals are to be estimated

Sim.M3 <- function(SimRuns, SampleSize, k.z, lam.cens, lam.0, beta.G, beta.X, beta.G.X, beta.Z, beta.G.Z, r, Zeval)
{
	Cover.Fi <- matrix(nrow=length(Zeval)^2, ncol=SimRuns)
	Cover.De <- matrix(nrow=length(Zeval)^2, ncol=SimRuns)
	Width.Fi <- matrix(nrow=length(Zeval)^2, ncol=SimRuns)
	Width.De <- matrix(nrow=length(Zeval)^2, ncol=SimRuns)

		
 for(rr in 1:SimRuns)
  {
	G <- sample(0:1, SampleSize, repl=T)
	Sigma <- matrix(r, nrow=k.z+1, ncol=k.z+1)
	diag(Sigma) <- 1
	Covs <- rmvnorm(SampleSize, mean = rep(0, k.z + 1), sigma = Sigma)
	
	hazard <- lam.0 * exp(beta.X*Covs[,1] + beta.G*G + beta.G.X*Covs[,1]*G + Covs[,2:dim(Covs)[2]]%*%matrix(beta.Z, ncol=1) + (G*Covs[,2:dim(Covs)[2]])%*%matrix(beta.G.Z, ncol=1))
	Ei <- rexp(SampleSize, rate = hazard)
	Ci <- rexp(SampleSize, lam.cens)
	Ti <- pmin(Ei, Ci)
	St <- as.numeric(Ei < Ci)

	DATA <- data.frame(Time=Ti, Status=St, Treat=G, Xvar=Covs[,1], ZVars=Covs[,-1])

	FI <- Fieller.M3.k2(Data=DATA, Timevar="Time", Statvar="Status", Treatvar="Treat", Xvar="Xvar", Zvar1="ZVars.1", Zvar2="ZVars.2", Zseq=Zeval)
	DE <- Delta.M3.k2(Data=DATA, Timevar="Time", Statvar="Status", Treatvar="Treat", Xvar="Xvar", Zvar1="ZVars.1", Zvar2="ZVars.2", Zseq=Zeval)

	Cover.Fi[, rr] <- as.numeric( (  (FI[, 3] < - (beta.G + beta.G.Z[1]*FI[, 1] + beta.G.Z[2]*FI[, 2]) / beta.G.X ) &
																		(FI[, 4] > - (beta.G + beta.G.Z[1]*FI[, 1] + beta.G.Z[2]*FI[, 2]) / beta.G.X ) ) | 
															 	( (FI[, 5] < - (beta.G + beta.G.Z[1]*FI[, 1] + beta.G.Z[2]*FI[, 2]) / beta.G.X ) &
															 			(FI[, 6] > - (beta.G + beta.G.Z[1]*FI[, 1] + beta.G.Z[2]*FI[, 2]) / beta.G.X) ) )
	
	Cover.De[, rr] <- as.numeric(  (DE[, 3] < - (beta.G + beta.G.Z[1]*DE[, 1] + beta.G.Z[2]*DE[, 2]) / beta.G.X ) &
																 (DE[, 4] > - (beta.G + beta.G.Z[1]*DE[, 1] + beta.G.Z[2]*DE[, 2]) / beta.G.X ))
	
	Width.Fi[, rr] <- (FI[, 4] - FI[, 3]) + (FI[, 6] - FI[, 5])
	Width.De[, rr] <-  DE[, 4] - DE[, 3]
	
  cat(rr, "/", sep="")
 }

 CoverMat <- cbind(FI[,1], FI[,2], apply(Cover.Fi, 1, mean), apply(Cover.De, 1, mean))
 WidthMat <- cbind(FI[,1], FI[,2], apply(Width.Fi, 1, median), apply(Width.Fi, 1, function(x) quantile(x, prob=0.1)), apply(Width.Fi, 1, function(x) quantile(x, prob=0.9)),
 																	 apply(Width.De, 1, median), apply(Width.De, 1, function(x) quantile(x, prob=0.1)), apply(Width.De, 1, function(x) quantile(x, prob=0.9)))
 colnames(CoverMat) <- c("Z_1", "Z_2", "Fieller", "Delta")
 colnames(WidthMat) <- c("Z_1", "Z_2", "Fieller_median", "Fieller_10th_perc", "Fieller_90th_perc", "Delta_median", "Delta_10th_perc", "Delta_90th_perc")

 Prop.infi.Fieller <- apply(Width.Fi, 1, function(x) mean(x==Inf))
 
 return(list(Coverage=CoverMat, Width=WidthMat, Fieller.prop.inf=Prop.infi.Fieller))
}

# A print funtion to organize the output
print.Sc.M3 <- function(Sim.Scen.M3)
{
	cat("Confidence interval coverage in dependence of z1 and z2:\n")
	print(Sim.Scen.M3$Coverage)
	
	cat("\nConfidence interval width in dependence of z1 and z2:\n")
	print(round(Sim.Scen.M3$Width, 2))
	
	cat("\nProportion of infinite intervals using Fieller's approach:", Sim.Scen.M3$Fieller.prop.inf[1])
}

# For example the fifth scenario for Model III can be initiated by:
M3.Sc5 <- Sim.M3(SimRuns=2000, SampleSize=1000, k.z=2, lam.cens=0.3, lam.0=1, beta.G=0, beta.X=log(1.25), beta.G.X=log(1.4), beta.Z=c(log(1.25),log(1.25)), 
							 beta.G.Z=c(log(1.2),log(1.2)), r=0, Zeval=qnorm(p = c(0.05, 0.25, 0.50, 0.75, 0.95)))
print.Sc.M3(M3.Sc5)

